--
--
-- control module (simulates SPIM control module)
--
--
library synth;
use synth.stdsynth.ALL;




entity control is

   generic(Tpd : Time := 1000PS);


   port( signal Op : in vlbit_1D(5 downto 0);

		signal RegDst : out vlbit;
		signal ALUSrc : out vlbit;
		signal MemtoReg : out vlbit;
		signal RegWrite : out vlbit;
		signal MemRead : out vlbit;
		signal MemWrite : out vlbit;
		signal Branch : out vlbit;
		signal ALUop0 : out vlbit;
		signal ALUop1 : out vlbit;
		signal phi1,phi2: in vlbit);
		
		
end control;


--
-- SPIM control architecture
--

architecture behaviour of control is
   
   signal  Rformat, Lw, Sw, Beq : vlbit;


begin           -- behaviour of SPIM control
------------------------------------------------------------------------------

   

   

		Rformat <= ((NOT Op(5)) AND (NOT Op(4)) AND (NOT Op(3)) AND
				   (NOT Op(2)) AND (NOT Op(1)) AND (NOT Op(0)));

		Lw      <= (    Op(5)) AND (NOT Op(4)) AND (NOT Op(3)) AND
				   (NOT Op(2)) AND (    Op(1)) AND (    Op(0));

		Sw      <= (    Op(5)) AND (NOT Op(4)) AND (    Op(3)) AND
				   (NOT Op(2)) AND (    Op(1)) AND (    Op(0));

		Beq     <= (NOT Op(5)) AND (NOT Op(4)) AND (NOT Op(3)) AND
				   (    Op(2)) AND (NOT Op(1)) AND (NOT Op(0));
   
		RegDst <=  Rformat after Tpd;
		ALUSrc <=  Lw or Sw after 2*Tpd;
		MemtoReg <=  Lw after Tpd;
		RegWrite <=  Rformat or Lw after 2*Tpd;
		MemRead <=  Lw after Tpd;
		MemWrite <=  Sw after Tpd; 
		Branch <=  Beq after Tpd;
		ALUOp1 <=  Rformat after Tpd;
		ALUOp0 <=  Beq after Tpd; 

   

end behaviour;

