--
--
-- Execute module (simulates SPIM (ALU) Execute module)
--
--
library synth;
use synth.stdsynth.all;

entity  Execute is

	generic(Tpd : Time := 1000PS);

	port(Readdata1 : in vlbit_1d(7 downto 0);
Readdata2 : in vlbit_1d(7 downto 0);
Extend : in vlbit_1d(7 downto 0);
Func : in vlbit_1d(5 downto 0);
ALUOp0 : in vlbit;
ALUOp1 : in vlbit; 
ALUSrc : in vlbit;
Zero : out vlbit;
ALUResult : out vlbit_1d(7 downto 0);
ADDResult : out vlbit_1d(7 downto 0);
PCadd : in vlbit_1d(7 downto 0);
phi1,phi2: in vlbit);

end Execute;

architecture behaviour of Execute is


 
Signal Ainput, Binput : vlbit_1d(7 downto 0);
Signal ResMux,Addout : vlbit_1d(8 downto 0);
Signal BAddout : vlbit_1d(8 downto 0);
Signal ALUctl: vlbit_1d(2 downto 0);
	begin
	 Ainput <= Readdata1;
	 Binput <= Readdata2 WHEN ((ALUSrc='0') AND (ALUctl/=B"11X"))
		ELSE Comp2c(Readdata2) WHEN ((AluSrc='0') AND (ALUctl=B"11X")) 
		ELSE Extend(7 downto 0);
	 ALUctl(0) <= (Func(0) OR Func(3)) AND ALUOp1;
	 ALUctl(1) <= (NOT Func(2)) OR (NOT ALUOp1);
	 ALUctl(2) <= (Func(1) AND ALUOp1) OR ALUOp0;
	 Addout <= add2c(Ainput,Binput);
	 with ALUctl select        
-- Select ALU operation
		 ResMux <=
-- ALU performs ALUresult = bus_A AND bus_B
		 '0' & (Ainput AND Binput) WHEN "000",
-- ALU performs ALUresult = bus_A OR bus_B
		 '0' & (Ainput OR Binput)  WHEN "001", 
-- ALU performs ALUresult = bus_a + bus_b
		 Addout WHEN "010",
-- ALU performs SLT
		 '0' & X"00" WHEN "011",
-- ALU performs ?
		 '0' & X"00" WHEN "100",
-- ALU performs ?
		 '0' & X"00" WHEN "101",
-- ALU performs ALUresult = bus_A -bus_B
		 Addout WHEN "110",
-- ALU performs SLT
		 '0' & X"00" WHEN "111";
-- Generate Zero Flag
	Zero <= boo2vlb(ResMux(7 downto 0) = X"00") after 6*Tpd;    
-- Select ALU output        
	ALUresult <= B"0000000" & ResMux(8) WHEN ALUctl="111" 
	ELSE ResMux(7 downto 0) after 5*Tpd;
-- Adder for Branch Address
	BAddout<=addum(PCadd,shiftlum(Extend(7 downto 0),2));
	ADDresult <= BAddout(7 downto 0) after 4*Tpd;        

end behaviour;

