package dffxtra is
-- dffl_v is an eight bit d flip flop with a parallel load feature
procedure dffl_v(signal data,loaddata : in vlbit_1d(7 downto 0); 
signal clock,load : in vlbit; 
signal qout :inout vlbit_1d(7 downto 0));
end;
package body dffxtra is
procedure dffl_v(signal data,loaddata : in vlbit_1d(7 downto 0); 
signal clock,load : in vlbit; 
signal qout :inout vlbit_1d(7 downto 0))
is
begin
 loop
 wait until (prising(clock) OR (load='1'));
 if load='1' then
  qout<=loaddata;
 else
  qout<=data;
 end if;
end loop;
end;
end dffxtra;
--
--
--  Idecode module (provides the register file for the SPIM computer)

library synth;
use synth.stdsynth.all;
use work.dffxtra.all;

entity Idecode is

	generic(Tpd : Time := 1000PS);
	port(rr1d_bus : out vlbit_1d(7 downto 0);
	rr1a_bus : in vlbit_1d(4 downto 0);
	rr2d_bus : out vlbit_1d(7 downto 0);
	rr2a_bus : in vlbit_1d(4 downto 0);
	wrd_bus : in vlbit_1d(7 downto 0);
	wra_bus : in vlbit_1d(4 downto 0);
	wra2_bus : in vlbit_1d(4 downto 0);
	RegWrite : in vlbit;
	RegDst : in vlbit;
	Ivalue : in vlbit_1d(15 downto 0);
	Extend : out vlbit_1d(7 downto 0);
		phi1,phi2,reset: in vlbit);

end Idecode;

--
-- Idecode architecture
--

architecture behaviour of Idecode is


  

signal wraddress : vlbit_1d(4 downto 0);
signal reg1,reg2,reg3,reg4,reg5,reg6,reg7: vlbit_1d(7 downto 0);
signal mrr1d_bus,mrr2d_bus: vlbit_1d(7 downto 0);
signal muxreg1,muxreg2,muxreg3,muxreg4: vlbit_1d(7 downto 0);
signal muxreg5,muxreg6,muxreg7: vlbit_1d(7 downto 0);
signal reg1wr,reg2wr,reg3wr,reg4wr: vlbit;
signal reg5wr,reg6wr,reg7wr: vlbit;
	begin
-- Read Register Operations
		WITH rr1a_bus(2 downto 0) SELECT
		mrr1d_bus <= X"00"  WHEN "000", 
		reg1 WHEN "001",
		reg2 WHEN "010",
		reg3 WHEN "011",
		reg4 WHEN "100",
		reg5 WHEN "101",
		reg6 WHEN "110",
		reg7 WHEN "111";
 
		 
		WITH rr2a_bus(2 downto 0) SELECT
		mrr2d_bus <= X"00"  WHEN "000", 
		reg1 WHEN "001",
		reg2 WHEN "010",
		reg3 WHEN "011",
		reg4 WHEN "100",
		reg5 WHEN "101",
		reg6 WHEN "110",
		reg7 WHEN "111";
		
		rr1d_bus <= mrr1d_bus After 5*Tpd;
		rr2d_bus <= mrr2d_bus After 5*Tpd;
-- Mux for Register Write Address
		wraddress <= wra_bus WHEN RegDst='1'     -- write address mux
		ELSE wra2_bus;
-- Register loaded during write back?
-- Note: Initial values in registers set to register number during reset 
-- to provide simple test data for MIPS programs
muxreg1 <=reg1 WHEN reg1wr='0' ELSE wrd_bus;
reg1wr <= boo2vlb((wraddress="00001") AND (RegWrite='1'));
dffl_v(muxreg1,X"01",phi2,reset,reg1);
muxreg2 <=reg2 WHEN reg2wr='0' ELSE wrd_bus;
reg2wr <= boo2vlb((wraddress="00010") AND (RegWrite='1'));
dffl_v(muxreg2,X"02",phi2,reset,reg2);
muxreg3 <=reg3 WHEN reg3wr='0' ELSE wrd_bus;
reg3wr <= boo2vlb((wraddress="00011") AND (RegWrite='1'));
dffl_v(muxreg3,X"03",phi2,reset,reg3);
muxreg4 <=reg4 WHEN reg4wr='0' ELSE wrd_bus;
reg4wr <= boo2vlb((wraddress="00100") AND (RegWrite='1'));
dffl_v(muxreg4,X"04",phi2,reset,reg4);
muxreg5 <=reg5 WHEN reg5wr='0' ELSE wrd_bus;
reg5wr <= boo2vlb((wraddress="00101") AND (RegWrite='1'));
dffl_v(muxreg5,X"05",phi2,reset,reg5);
muxreg6 <=reg6 WHEN reg6wr='0' ELSE wrd_bus;
reg6wr <= boo2vlb((wraddress="00110") AND (RegWrite='1'));
dffl_v(muxreg6,X"06",phi2,reset,reg6);
muxreg7 <=reg7 WHEN reg7wr='0' ELSE wrd_bus;
reg7wr <= boo2vlb((wraddress="00111") AND (RegWrite='1'));
dffl_v(muxreg7,X"07",phi2,reset,reg7);
-- Sign Extend Unit  (NOTE: not needed for 8-bit data path model)
--        Extend(15 downto 0) <= Ivalue;  -- Sign Extend 16 to 32 bits
--        Extend(31 downto 16) <= X"FFFF" WHEN Ivalue(15)='1' 
--        ELSE X"0000" after Tpd;
		  Extend <= Ivalue(7 downto 0) after Tpd;


end behaviour;

