--
--
-- Ifetch module (provides the PC and instruction memory for the SPIM computer)

library synth;
use synth.stdsynth.ALL;

entity Ifetch is
	generic(Tpd: Time := 1000PS);
	port(signal Instruction : out vlbit_1D(31 downto 0);
	signal PCadd : out  vlbit_1D(7 downto 0);
	signal Addresult : in vlbit_1D(7 downto 0);
	signal Branch : in vlbit;
	signal phi1, phi2, reset : in vlbit;
	signal Zero : in vlbit;
	signal PCout : out vlbit_1d(7 downto 0));

end Ifetch;

--
-- Ifetch architecture
--
library synth;
use synth.stdsynth.all;

architecture behaviour of Ifetch is


	  signal PC : vlbit_1D(7 downto 0);
	  signal PCtemp : vlbit_1D(7 downto 0);
	  signal PCaddtemp : vlbit_1D(8 downto 0);
-- Insert SPIM Machine Language Test Program Here
	  constant mem0 : vlbit_1D(31 downto 0) := (
-- Field   | op | rs | rt | rd ?addr/immed|
	 B"10001100000000100000000000000000"); -- lw $2,0
	  constant mem1 : vlbit_1D(31 downto 0) := (
	 B"10001100000000110000000000000001"); -- lw $3,1
	  constant mem2 : vlbit_1D(31 downto 0) := (
	 B"00000000010000110000100000100000"); -- add $1,$2,$3
	  constant mem3 : vlbit_1D(31 downto 0) := (
	 B"10101100000000010000000000000011"); -- sw $1,3
	  constant mem4 : vlbit_1D(31 downto 0) := (
	B"00010000001000101111111111111111"); -- beq $1,$2,-4
	  constant mem5 : vlbit_1D(31 downto 0) := (
	 B"00010000001000011111111111111111"); -- beq $1,$1,-4
	  constant mem6 : vlbit_1D(31 downto 0) := (
	 B"00000000000000000000000000000000");                                    
	  constant mem7 : vlbit_1D(31 downto 0) := (
	 B"00000000000000000000000000000000");                                   
  

	begin
-- Increment PC by 4        
	PCout <= PC;
	PCaddtemp <= addum(PC,X"04");
	PCadd <= PCaddtemp(7 downto 0) AFTER 2*Tpd;
-- Mux for Branch Address or Next Address        
	PCtemp <= Addresult WHEN ((Branch='1') AND (Zero='1')) 
	ELSE PCaddtemp(7 downto 0);
-- Load next PC
		dffc_v(PCtemp, reset, phi2, PC);

-- Fetch Instruction from memory     
	  with PC(4 downto 2) select 
	  instruction <=
	  mem0 WHEN "000",
	  mem1 WHEN "001",
	  mem2 WHEN "010",
	  mem3 WHEN "011",
	  mem4 WHEN "100",
	  mem5 WHEN "101",
	  mem6 WHEN "110",
	  mem7 when "111";

end behaviour;

